/****************************************************************************
* MeshLab                                                           o o     *
* A versatile mesh processing toolbox                             o     o   *
*                                                                _   O  _   *
* Copyright(C) 2005                                                \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/

#include "filter_remesh.h"
#include "split.h"
#include <QtScript>

// Constructor usually performs only two simple tasks of filling the two lists 
//  - typeList: with all the possible id of the filtering actions
//  - actionList with the corresponding actions. If you want to add icons to your filtering actions you can do here by construction the QActions accordingly

ExtraSamplePlugin::ExtraSamplePlugin() 
{ 
    typeList << FP_MESH_REFINE_WITH_POINTS;
  
  foreach(FilterIDType tt , types())
	  actionList << new QAction(filterName(tt), this);
}

// ST() must return the very short string describing each filtering action 
// (this string is used also to define the menu entry)
QString ExtraSamplePlugin::filterName(FilterIDType filterId) const
{
  switch(filterId) {
        case FP_MESH_REFINE_WITH_POINTS :  return QString("Refine a mesh with a given set of points");
		default : assert(0); 
	}
  return QString();
}

// Info() must return the longer string describing each filtering action 
// (this string is used in the About plugin dialog)
 QString ExtraSamplePlugin::filterInfo(FilterIDType filterId) const
{
  switch(filterId) {
        case FP_MESH_REFINE_WITH_POINTS :  return QString("Creates a new mesh that is composed by all the original triangles splitted by a given set of points.");
		default : assert(0); 
	}
	return QString("Unknown Filter");
}

// The FilterClass describes in which generic class of filters it fits. 
// This choice affect the submenu in which each filter will be placed 
// More than a single class can be choosen.
ExtraSamplePlugin::FilterClass ExtraSamplePlugin::getClass(QAction *a)
{
  switch(ID(a))
	{
		case FP_MESH_REFINE_WITH_POINTS :  return MeshFilterInterface::Remeshing;
		default : assert(0); 
	}
	return MeshFilterInterface::Generic;
}

// This function define the needed parameters for each filter. Return true if the filter has some parameters
// it is called every time, so you can set the default value of parameters according to the mesh
// For each parameter you need to define, 
// - the name of the parameter, 
// - the string shown in the dialog 
// - the default value
// - a possibly long string describing the meaning of that parameter (shown as a popup help in the dialog)
void ExtraSamplePlugin::initParameterSet(QAction *action,MeshDocument &md, RichParameterSet & parlst)
{
	 switch(ID(action))	 {
		case FP_MESH_REFINE_WITH_POINTS :
	   parlst.addParam(new RichMesh("BaseMesh",md.mm(), &md,"Base Mesh","The mesh whose triangles are splitted by."));
	   parlst.addParam(new RichMesh("PointMesh",md.mm(), &md,"Point Mesh","The set of points that are used to split the first mesh"));
		break;
		default : assert(0); 
	}
}

// The Real Core Function doing the actual mesh processing.
// Move Vertex of a random quantity
bool ExtraSamplePlugin::applyFilter(QAction *filter, MeshDocument &md, RichParameterSet & par, vcg::CallBackPos *cb)
{
  switch(ID(filter))
  {
  case FP_MESH_REFINE_WITH_POINTS :
  {
    MeshModel *bm = par.getMesh("BaseMesh");
    bm->updateDataMask(MeshModel::MM_FACEMARK);
    MeshModel *pm = par.getMesh("PointMesh");
    MeshModel *out = md.addNewMesh("","Splitted");
    vcg::Split::Mesh<CMeshO>(bm->cm,pm->cm,out->cm);
    Log("Success");
    return true;
  }
  default: assert(0);
  }
  return false;
}

QString ExtraSamplePlugin::filterScriptFunctionName( FilterIDType filterID )
{
	switch(filterID) {
		case FP_MESH_REFINE_WITH_POINTS :  return QString("randomVerticesDisplacement");
		default : assert(0); 
	}
	return QString();
}

Q_EXPORT_PLUGIN(ExtraSamplePlugin)
